﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace WMS_v10
{
    public partial class Form1 : Form
    {
        private LayerOverlay CultureOverlay
        {
            get { return (LayerOverlay)winformsMap1.Overlays["Culture"]; }
        }

        private Proj4Projection _proj;
        private static int EPSG = 32615;

        public Form1()
        {
            InitializeComponent();

            SetupMap();
        }


        private void SetupMap()
        {
            winformsMap1.MapUnit = GeographyUnit.Meter;

            LayerOverlay lo = new LayerOverlay();
            lo.Name = "Culture";

            winformsMap1.Overlays.Add("Culture", lo);

            _proj = new Proj4Projection();
            _proj.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(EPSG);
            _proj.ExternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(EPSG);


            ShapeFileFeatureLayer shp = new ShapeFileFeatureLayer(@"../../../Data/Hammer S-R.shp");
            //ShapeFileFeatureLayer shp = new ShapeFileFeatureLayer(@"../../../Data/garlock_SRTM_30m_boundary.shp");
            shp.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            shp.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            shp.Open();
            RectangleShape rect = shp.GetBoundingBox();
            shp.Close();
            winformsMap1.CurrentExtent = rect;

            lo.Layers.Add("boundary", shp);

            winformsMap1.Refresh();

            winformsMap1.MouseMove += winformsMap1_MouseMove;
        }

        void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            PointShape WorldPointR = ExtentHelper.ToWorldCoordinate
                (winformsMap1.CurrentExtent, e.X, e.Y, winformsMap1.Width, winformsMap1.Height);

            label1.Text = string.Format("X: {0}m, Y: {1}m",
                Math.Round(WorldPointR.X, 0), Math.Round(WorldPointR.Y, 0));


        }

        private void btnWMS_Click(object sender, EventArgs e)
        {
            WMSService wms = new WMSService("Weather", new Uri(@"https://mesonet.agron.iastate.edu/cgi-bin/wms/us/mrms.cgi?VERSION=1.1.1"), null);
            if (wms.ShowDialog() != DialogResult.OK) return;

            WmsRasterLayer wmsfl = new WmsRasterLayer(wms.URI);
            wmsfl.Name = wms.Name;

            string cred = wms.URI.UserInfo;
            if (cred != "")
            {
                string[] values = cred.Split(':');
                wmsfl.Credentials = new NetworkCredential(values[0], values[1]);
            }

            wmsfl.Open();
            wmsfl.UpperThreshold = double.MaxValue;
            wmsfl.LowerThreshold = 0;
            wmsfl.TimeoutInSecond = 60;

            foreach (string layer in wms.Layers)
            {
                wmsfl.ActiveLayerNames.Add(layer);
            }

            //Collection<string> styles = wmsfl.GetServerStyleNames();
            //foreach (string style in styles)
            //{
            //    wmsfl.ActiveStyleNames.Add(style);
            //}



            // If the CRS is available at the server, 
            // use it because they might not support on the fly reprojection
            // actually, this doesn't seem to be the case, only 4326 works well...
            Collection<string> crs = wmsfl.GetServerCrss();
            if (!crs.Contains("EPSG:" + 4326))
            {
                if (crs.Contains("EPSG:" + EPSG))
                    wmsfl.Crs = "EPSG:" + EPSG;
            }

            int code = Convert.ToInt32(wmsfl.Crs.Replace("EPSG:", ""));
            string proj4 = Proj4Projection.GetEpsgParametersString(code);
            Proj4Projection proj = new Proj4Projection();
            proj.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(code);
            proj.ExternalProjectionParametersString = _proj.ExternalProjectionParametersString;

            proj.Open();

            wmsfl.ImageSource.Projection = proj;
            wmsfl.Close();

            CultureOverlay.Layers.Add(wms.Name, wmsfl);
            CultureOverlay.Layers.MoveToBottom(wmsfl);
            winformsMap1.Refresh();
        }


    }
}

