﻿using DevExpress.XtraTreeList.Nodes;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Layers;

namespace WMS_v10
{
    public partial class WMSService : Form
    {
        public string Name { get; set; }
        public Uri URI { get; set; }
        public System.Net.WebProxy Proxy { get; set; }
        public Collection<string> Layers { get; set; }

        public WMSService(string name, Uri uri, System.Net.WebProxy proxy)
        {
            InitializeComponent();

            Name = name;
            URI = uri;
            Proxy = proxy;

            textName.Text = name;
            textURI.Text = uri.OriginalString;
            if (proxy != null)
                textProxy.Text = proxy.Address.ToString();
        }

        public WMSService()
        {
            InitializeComponent();
        }

        private void btnOkay_Click(object sender, EventArgs e)
        {
            if (textName.Text == "")
            {
                MessageBox.Show("Please enter a service name");
                return;
            }

            if (textURI.Text == "")
            {
                MessageBox.Show("Please enter a URI");
                return;
            }

            // Remove garbage from the end of the request and check for the version
            string uri = textURI.Text;
            int index = uri.IndexOf("=1.1.1");
            if (index == -1)
            {
                MessageBox.Show("URI should be for Version 1.1.1.\n\n" +
                    "Please modify the URI to point to this version", "Incorrect version",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            uri = uri.Substring(0, index + 6);

            Uri uriResult;
            bool result = Uri.TryCreate(uri, UriKind.Absolute, out uriResult)
                && (uriResult.Scheme == Uri.UriSchemeHttp || uriResult.Scheme == Uri.UriSchemeHttps);

            if (!result)
            {
                MessageBox.Show("This URI is not valid.  Please check for spelling and that the site is live.", 
                    "Invalid URI", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            Name = textName.Text;

            // Add username and password to the uri if needed
            if (textUsername.Text != "" && textPassword.Text != "")
            {
                Uri temp = new Uri(uri);
                if (temp.UserInfo == "")
                {
                    uri = uri.Replace("://", "://" + textUsername.Text + ":" + textPassword.Text + "@");
                }
            }

            URI = new Uri(uri);

            if (textProxy.Text != "")
                Proxy = new System.Net.WebProxy(textProxy.Text);

            Layers = new Collection<string>();
            foreach (TreeListNode node in treeList1.Nodes)
            {
                if (!node.Checked) continue;
                Layers.Add(node.GetDisplayText("Layers"));
            }

            if (Layers.Count == 0)
            {
                MessageBox.Show("You must select at least one layer.  " + 
                    "Click Select Layers for a full list on the site.",
                    "No layer selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            this.DialogResult = DialogResult.OK;
            this.Close();

        }

        private void btnLayers_Click(object sender, EventArgs e)
        {
            
            Uri uriResult;

            // Remove garbage from the end of the request and check for the version
            string uri = textURI.Text;
            int index = uri.IndexOf("=1.1.1");
            if (index == -1)
            {
                MessageBox.Show("URI should be for Version 1.1.1.\n\n" +
                    "Please modify the URI to point to this version", "Incorrect version",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            uri = uri.Substring(0, index + 6);


            bool result = Uri.TryCreate(uri, UriKind.Absolute, out uriResult)
                && (uriResult.Scheme == Uri.UriSchemeHttp || uriResult.Scheme == Uri.UriSchemeHttps);

            if (!result)
            {
                MessageBox.Show("This URI is not valid.  Please check for spelling and that the site is live.", 
                    "Invalid URI", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            WmsRasterLayer wmsfl = new WmsRasterLayer(uriResult);
            string cred = uriResult.UserInfo;
            if (cred != "")
            {
                string[] values = cred.Split(':');
                wmsfl.Credentials = new NetworkCredential(values[0], values[1]);
            }

            if (textUsername.Text != "" && textPassword.Text != "")
            {
                wmsfl.Credentials = new NetworkCredential(textUsername.Text, textPassword.Text);
            }

            try
            {
                wmsfl.Open();
                Collection<string> layers = wmsfl.GetServerLayerNames();
                wmsfl.Close();

                foreach (string layer in layers)
                {
                    treeList1.AppendNode(new object[] { layer }, null);
                }
            }
            catch(Exception ex)
            {
                MessageBox.Show("Error opening service.\n\n" + ex.Message,
                    "Wms error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }


        }
    }
}
